<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * アクションコントローラ ZFReaderのメイン
 * 
 * @author ishinao <ishinao@ishinao.net>
 * $Id: IndexController.php 72 2006-05-15 09:19:25Z ishinao $
 */

/**
 * アクションコントローラ基底クラスを読み込み
 */
Zend::loadClass('Zend_Controller_Action');

/**
 * アクションコントローラクラス
 * 
 */
class IndexController extends Zend_Controller_Action
{
    /**
     * デフォルト（＝トップページ）アクション
     */
    public function indexAction()
    {

    	$feedTable = new Feed(); // Feedテーブルクラス生成
    	$view = Zend::registry('view'); // registyからViewオブジェクトを取得
    	$view->feeds = $feedTable->fetchAll(); // すべてのフィード情報をセット
    	
    	echo $view->render('pages/index.php'); // ページテンプレートを表示

    }

    /**
     * フィードを読むアクション
     * 
     * @params integer id = null 表示するフィードのID。省略時は表示しない。
     */
    public function readAction()
    {

    	$feedTable = new Feed(); // Feedテーブルクラスを生成
    	$view = Zend::registry('view'); // registryからViewオブジェクトを取得
    	$view->feeds = $feedTable->fetchAll(); // すべてのフィード情報をセット
    	
    	$id = $this->_getParam('id'); // アクションパラメータからフィードIDを取得

    	if ($id) { // フィードIDがセットされていたら

    		$view->feedInfo = $feedTable->find($id); // フィード情報をセット

	    	$itemTable = new Item(); // Itemテーブルクラスを生成
	    	$view->items = $itemTable->getRecentItems($id); // 新着アイテムをセット

    	}
    	
    	echo $view->render('pages/read.php'); // ページテンプレートを表示

    }

    /**
     * フィード追加アクション
     * 
     * @params string url = null フィードURL。省略時はREQUESTパラメータから情報を取得。
     */
    public function addAction()
    {

    	$request = Zend::registry('request'); // REQUESTパラメータオブジェクトを取得
    	$url = $this->_getParam('url'); // アクションパラメータからフィードURLを取得

    	if ($url == '') { // アクションパラメータからフィードURLを取得できない
    		$url = $request->getRaw('url'); // REQUESTパラメータからURLを取得
    	}

    	if ($url == '') { // アクションパラメータ、REQUESTパラメータのどちらでもURLが指定されていない
    		return $this->_forward('index', 'addForm'); // URL入力フォームアクションへ遷移
    	}

    	$feedTable = new Feed(); // Feedテーブルクラスを生成
    	
    	try {

    		$feedId = $feedTable->add($url); // フィードURLを追加
			Zend_Log::log('add url: ' . $url);
		

			// メッセージを表示してトップにリダイレクト
	    	$this->_forward('system', 'redirect', array(
	    		'message' => 'フィードを登録しました。',
	    		'url' => ZFReader_Controller_Router::getUrl('/'),
	    	));
	    	return;
	    	
    	} catch (FeedAlreadyRegisteredException $e) {
    		
    		$this->_forward('system', 'error', array(
				'message' => 'URLはすでに登録されています: ' . $url
			));
    		return;
    		
    	} catch (Exception $e) {

			// Zend_Feedレベルの例外
			Zend_Log::log('add url failed: ' . $url . ' ' . $e->getMessage()); // ログに例外の内容を残す
			
			// ユーザーには通常のエラーメッセージを返す
    		$this->_forward('system', 'error', array(
				'message' => 'URLの登録に失敗しました: ' . $url
			));
    		return;

    	}

    }
    
    /**
     * フィード追加フォームアクション
     * 
     * @param string url フィードURL。省略可。
     * @param string message メッセージ。省略可。
     */
    public function addFormAction()
    {

    	$view = Zend::registry('view'); // Viewオブジェクトを取得

    	$view->url = $this->_getParam('url'); // URLパラメータをセット
    	$view->message = $this->_getParam('message'); // メッセージパラメータをセット

    	echo $view->render('pages/addForm.php'); // テンプレートを出力

    }

    /**
     * フィード削除アクション
     * 
     * @param integer id フィードID
     */
    public function removeAction()
    {

    	$id = $this->_getParam('id'); // フィードIDを取得

    	$feedTable = new Feed(); // Feedテーブルクラスを生成
    	$db = $feedTable->getAdapter(); // Zend_Db_Adapterクラスを取得（quoteIntoを使うため）
    	
    	if (!$feedTable->exists($id)) { // フィードIDが不正な場合
    	
    		// エラーページに遷移
    		$this->_forward('system', 'error', array(
				'message' => '指定されたフィードが見つかりません: ' . $id
			));
    		return;
    	}
    	
    	$feedTable->delete($db->quoteInto('id = ?', $id)); // フィードを削除
    	
    	$itemTable = new Item(); // Itemテーブルクラスを生成
    	$itemTable->delete($db->quoteInto('feed_id = ?', $id)); // アイテムを削除

		// メッセージを表示してトップにリダイレクト
    	$this->_forward('system', 'redirect', array(
    		'message' => 'フィードを削除しました。',
    		'url' => ZFReader_Controller_Router::getUrl('/'),
    	));

    }
    
    /**
     * フィード情報編集フォームアクション
     * 
     * @param integer id フィードID
     */
    public function editAction()
    {

    	$id = $this->_getParam('id'); // フィードIDを取得

    	$feedTable = new Feed(); // Feedテーブルクラスを生成

    	if (!$feedTable->exists($id)) {
    		
    		// フィードIDが不正ならエラーページに遷移
    		$this->_forward('system', 'error', array(
				'message' => '指定されたフィードが見つかりません: ' . $id
			));
    		return;

    	}
    	
    	$server = Zend::registry('server'); // $_SERVER入力フィルタ
    	$post = Zend::registry('post'); // $_POST入力フィルタ
    	$view = Zend::registry('view'); // Viewオブジェクトを取得

		$feed= $feedTable->find($id); // フィード情報を取得
		
		$view->feedid = $feed->id;
		$view->feedurl = $feed->feedurl;
    	$view->url = $feed->url;
    	$view->title = $feed->title; 

    	if ($server->getRaw('REQUEST_METHOD') == 'POST') { // フォームからのPOSTだったら

			$url = trim($post->getRaw('url')); // URLを取得
			$title = trim($post->getRaw('title')); // タイトルを取得

			$errors = array();

			if ($title == '') { // タイトル必須
				$errors[] = 'タイトルを入力してください';
			}

			Zend::loadClass('Zend_Uri');

			if ($url == '') { // URL必須

				$errors[] = 'URLを入力してください';

			} else if (!Zend_Uri::check($url)) { // URLとして正しい形式か？
			
				/**
				 * Zend_Input::isUriが実装済みならば（現時点では未実装）、
				 * URLの正当性チェックは
				 * 
				 * if (!$url = $post->testUri('url')) {
				 *   $errors[] = 'URLが不正です';
				 * }
				 * 
				 * なんて感じで書けるようになるはず。
				 */
				$errors[] = 'URLが不正です: ' . $url;

			}		

			if (count($errors) == 0) {

	    		$feed = $feedTable->find($id); // フィード情報を読み込み

	    		$feed->url = $url; // URLを更新
	    		$feed->title = $title; // タイトルを更新
	
	    		$feed->save(); // DBに保存
	    		
	    		// 投稿確認メッセージ＋リダイレクトアクションに遷移
	    		$this->_forward('system', 'redirect', array(
					'url' => ZFReader_Controller_Router::getUrl('/'),
					'message' => 'フィード情報を更新しました: ' . $id,
				));
	    		return;

			} else {
				
				$view->url = $url; // URLパラメータをセット
				$view->title = $title; // タイトルパラメータをセット
				$view->errorMessages = $errors; // エラーメッセージパラメータをセット
			}

    	}

    	echo $view->render('pages/editForm.php'); // ページテンプレートを表示

    }
    
    /**
     * フィードアイテム情報更新アクション
     * 
     * @param integer id フィードID。省略されている場合はすべてのフィードを更新する
     */
    public function crawlAction()
    {

    	$id = $this->_getParam('id'); // フィードIDを取得

    	$feedTable = new Feed(); // Feedテーブルクラス生成

    	try {
    		
	    	$feedTable->crawlFeed($id); // フィード情報の更新

    	} catch (Feed_InvalidIdException $e) {

    		$this->_forward('system', 'error', array(
    			'message' => '不正なフィードIDが指定されました: ' . $id
    		));
    		return;

    	}
    	
    	// 更新したフィード表示ページにリダイレクト
    	$this->_redirect(ZFReader_Controller_Router::getUrl('/read/' . $id));

    }
    
    /**
     * すべてのフィードアイテム情報を更新するアクション
     */
    public function crawlAllAction()
    {

    	$feedTable = new Feed(); // Feedテーブルクラス生成
    	$feeds = $feedTable->fetchAll(); // 登録されているすべてのフィード情報を取得

		// すべてのフィードに対してフィード情報の更新を行う
    	foreach ($feeds as $feed) {

			// 最後に情報取得してから10分以内の場合は取得しにいかない
    		if (time() - strtotime($feed->crawled) < 600) {continue;}
			
    		$feedTable->crawlFeed($feed->id); // 最新情報を取得

    	}
		// フィードを読むページにリダイレクト
    	$this->_redirect(ZFReader_Controller_Router::getUrl('/read'));

    }

    /**
     * AJAX用フィード情報取得アクション
     * 
     * XmlHttpRequestでフィード情報を取得するためのアクション。
     * array(
     *   'feed' => [フィードの情報],
     *   'items' => [フィードの最新アイテム一覧],
     * )
     * というPHPの配列データをJSON形式で返す。
     * 
     * @param integer id フィードID
     */
    public function feedInfoAction()
    {

    	$id = $this->_getParam('id'); // フィードIDを取得
    	
    	$result = array();

    	$feedTable = new Feed(); //Feedテーブルクラスを生成
    	$result['feed'] = $feedTable->find($id)->toArray(); // フィードに関する情報

    	$itemTable = new Item(); // Itemテーブルクラスを生成
    	$items = $itemTable->getRecentItems($id); // 最新アイテム一覧を取得

		// Zend_Db_Table_RowからPHP5の配列に変換
    	$result['items'] = array();
    	foreach ($items as $item) {
    		$result['items'][] = $item->toArray();
    	}

    	Zend::loadClass('Zend_Json'); // Zend_Jsonクラスを読み込み
    	echo Zend_Json::encode($result); // JSON形式にエンコードした結果を出力

    }
}

?>
