<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * Itemテーブルクラス
 * 
 * @author ishinao <ishinao@ishinao.net>
 * $Id: Item.php 73 2006-05-16 06:22:36Z ishinao $
 */

/**
 * Zend_Db_Tableクラスを読み込み
 */
Zend::loadClass('Zend_Db_Table');

/**
 * Itemクラスで生成する例外クラスを定義
 */
class Item_InvalidItemException extends Exception {} // 不正なアイテム 

/**
 * itemテーブルに対応するO/Rマッピングクラス
 */
class Item extends Zend_Db_Table
{
	/**
	 * insertメソッドをoverride
	 */
	public function insert(&$data)
	{

		$data['created'] = date('Y-m-d H:i:s');
		$data['modified'] = date('Y-m-d H:i:s');

		return parent::insert($data);

	}
	
	/**
	 * updateメソッドをoverride
	 */
	public function update(&$data, $where)
	{

		$data['modified'] = date('Y-m-d H:i:s');
		return parent::update($data, $where);

	}
	
	/**
	 * フィードアイテムを登録・更新する
	 * 
	 * Zend_Feedで取得したフィード情報を元にアイテムの情報を登録する。
	 * すでに登録済みのアイテム情報に関しては、更新を行う。
	 * 
	 * @param integer $feedId フィードID
	 * @param object $item Zend_Feedから取得したフィードアイテムオブジェクト
	 * @return integer 登録したアイテムID
	 */
	public function addItem($feedId, $item)
	{
		
		$db = $this->getAdapter(); // Zend_Db_Adapterオブジェクトを取得

		// URLを取得
		if ($item->link()) {

			$link = $item->link(); // RSSの場合

		} else if ($item->link['href']) { 

			$link = $item->link['href']; // Atomの場合

		} else {

			throw new Item_InvalidItemException('URLが見つかりません');

		}

		Zend::loadClass('Zend_Uri');
		if (!Zend_Uri::check($link)) {

			throw new Item_InvalidItemException('URLが不正です: ' . $link);

		}

		// すでに登録されているフィードID、アイテムの組み合わせを取得
		$row = $this->fetchRow(
			$db->quoteInto('feed_id = ?', $feedId) .
			' and ' .
			$db->quoteInto('url = ?', $link)
		);

		if (!$row->id) { // 登録されていなかった

			$row = $this->fetchNew(); // 新しいItem行を作成
			$row->feedId = $feedId; // フィードIDをセット
			$row->url = $link; // アイテムURLをセット

		}

		$row->title = $item->title(); // アイテムタイトルをセット
		
		if ($item->description()) {
			$description = $item->description();
		} else if ($item->content()) {
			$description = $item->content();
		} else if ($item->summary()) {
			$description = $item->summary();
		} else {
			$description = 'no summary';
		}
		$row->description = strip_tags($description); // アイテム概要をセット（タグを除去）

		// 日付情報をセット
		if ($item->pubDate()) {

			$row->contentDate = date('Y-m-d H:i:s', strtotime($item->pubDate()));
			
		} else if ($item->created()) {
			
			$row->contentDate = date('Y-m-d H:i:s', strtotime($item->created()));

		} else if ($row->contentDate == ''){

			// それ以外の日付形式対応は面倒なので
			// ひとまず初回アイテム情報取得日時を入れておく
			$row->contentDate = date('Y-m-d H:i:s');

		}

		return $row->save(); // アイテムIDを返す
	}
	
	/**
	 * 最新アイテム一覧を返す
	 * 
	 * @param integer $feedId フィードID。
	 * @param integer $limit  取得件数。デフォルトでは10件。
	 */
	public function getRecentItems($feedId, $limit = 10)
	{

		$db = $this->getAdapter(); // Zend_Db_Adapterオブジェクトを取得（quoteIntoのため）

		$where = $db->quoteInto('feed_id = ?', $feedId); // 検索条件
		$order = 'content_date desc'; // ソート条件

		return $this->fetchAll($where, $order, $limit); // 検索結果を返す

	}
}

?>
