<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * 起動ファイル
 *
 * @author ishinao <ishinao@ishinao.net>
 * $Id: index.php 72 2006-05-15 09:19:25Z ishinao $
 */

/**
 * ライブラリ、アプリケーションパスの設定
 *
 * もしもディレクトリ構成を変更する場合は、以下2行を変更する。
 */
define('LIBRARY_PATH', realpath('../library'));
define('APPLICATION_PATH', realpath('../application'));

/**
 * include_pathの設定
 */
set_include_path(implode(PATH_SEPARATOR, array(
    '.',
    LIBRARY_PATH, // Zend Framework library and other libraries
)));

/**
 * Zend Frameworkコアライブラリの読み込み
 */
require_once 'Zend.php';

/**
 * DBクラス生成
 *
 * DB設定を変更する場合は、下記引数を変更する。
 */
Zend::loadClass('Zend_Db');
$db = Zend_Db::factory('pdoSqlite', array(
    'dbname' => APPLICATION_PATH . '/sqlite.db'
));
Zend::register('db', $db);

/**
 * Zend_Db_Table初期化
 */
Zend::loadClass('Zend_Db_Table');
Zend_Db_Table::setDefaultAdapter($db);
require_once APPLICATION_PATH . '/models/Feed.php';
require_once APPLICATION_PATH . '/models/Item.php';

/**
 * 入力フィルターオブジェクトの生成
 *
 * 利用する入力値のフィルタオブジェクトを生成し、
 * Zend::registryに登録しておく。
 */
Zend::loadClass('Zend_Filter_Input');
Zend::register('post', new Zend_Filter_Input($_POST, false));
Zend::register('get', new Zend_Filter_Input($_GET, false));
Zend::register('request', new Zend_Filter_Input($_REQUEST, false));
Zend::register('server', new Zend_Filter_Input($_SERVER, false));

/**
 * ロガーオブジェクトの登録
 */
Zend::loadClass('Zend_Log');
Zend::loadClass('Zend_Log_Adapter_File');
Zend_Log::registerLogger(new Zend_Log_Adapter_File(APPLICATION_PATH . '/zfreader.log'));

/**
 * ビューオブジェクトの生成
 *
 * ここで一つのビューオブジェクトを生成しておき、各種設定を施した
 * 上で、Zend::registryに登録しておく。
 */
Zend::loadClass('Zend_View');
$view = new Zend_View();
$view->addScriptPath(APPLICATION_PATH . '/views');        //テンプレートディレクトリ設定
$view->addHelperPath(APPLICATION_PATH . '/helpers');      //ヘルパーディレクトリ追加
$view->addFilterPath(APPLICATION_PATH . '/filters');      //フィルターディレクトリ追加
$view->addFilter('tabToSpaces');
Zend::register('view', $view);

/**
 * フロントコントローラ
 */
Zend::loadClass('Zend_Controller_Front');
$controller = Zend_Controller_Front::getInstance();
$controller->setControllerDirectory(APPLICATION_PATH . '/controllers');

/**
 * 独自ルーターに差し替え
 */
Zend::loadClass('ZFReader_Controller_Router');
$controller->setRouter(new ZFReader_Controller_Router());

/**
 * プラグイン登録
 */
require_once APPLICATION_PATH . '/plugins/AuthPlugin.php';
$controller->registerPlugin(new AuthPlugin(APPLICATION_PATH . '/auth.txt')); //認証プラグイン

/**
 * フロントコントローラ実行
 */
$controller->dispatch();

?>
